@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
+
 from collections import defaultdict
 from copy import copy
 from datetime import datetime
@@ -33,7 +34,7 @@ def content_processor_jinja2(item):
             "You probably have to install it using `pip install Jinja2`."
         ).format(item=item.id))
 
-    loader = FileSystemLoader(searchpath=item.item_dir)
+    loader = FileSystemLoader(searchpath=[item.item_data_dir, item.item_dir])
     env = Environment(loader=loader)
 
     template = env.from_string(item._template_content.decode('utf-8'))
@@ -78,7 +79,7 @@ def content_processor_mako(item):
     template = Template(
         item._template_content,
         input_encoding='utf-8',
-        lookup=TemplateLookup(directories=[item.item_dir]),
+        lookup=TemplateLookup(directories=[item.item_data_dir, item.item_dir]),
         output_encoding=item.attributes['encoding'],
     )
     LOG.debug("{node}:{bundle}:{item}: rendering with Mako...".format(
@@ -248,9 +249,14 @@ class File(Item):
     @property
     def _template_content(self):
         if self.attributes['source'] is not None:
-            filename = join(self.item_dir, self.attributes['source'])
-            with open(filename) as f:
-                content = f.read()
+            filename = join(self.item_data_dir, self.attributes['source'])
+            if exists(filename):
+                with open(filename) as f:
+                    content = f.read()
+            else:
+                filename = join(self.item_dir, self.attributes['source'])
+                with open(filename) as f:
+                    content = f.read()
             return content
         else:
             return self.attributes['content']
@@ -268,6 +274,9 @@ class File(Item):
 
     @cached_property
     def template(self):
+        data_template = join(self.item_data_dir, self.attributes['source'])
+        if exists(data_template):
+            return data_template
         return join(self.item_dir, self.attributes['source'])
 
     def ask(self, status):
