@@ -12,10 +12,20 @@ from time import time
 from . import operations
 from .bundle import Bundle
 from .concurrency import WorkerPool
-from .deps import find_item, prepare_dependencies, remove_item_dependents, remove_dep_from_items, \
-    split_items_without_deps
-from .exceptions import ItemDependencyError, NodeAlreadyLockedException, NoSuchItem,\
-    RepositoryError
+from .deps import (
+    find_item,
+    prepare_dependencies,
+    remove_item_dependents,
+    remove_dep_from_items,
+    split_items_without_deps,
+)
+from .exceptions import (
+    ItemDependencyError,
+    NodeAlreadyLockedException,
+    NoSuchBundle,
+    NoSuchItem,
+    RepositoryError,
+)
 from .items import Item
 from .utils import cached_property, LOG, graph_for_items, names
 from .utils.text import mark_for_translation as _
@@ -36,8 +46,10 @@ class ApplyResult(object):
         self.fixed = 0
         self.skipped = 0
         self.failed = 0
+        self.profiling_info = []
 
-        for item_id, result in item_results:
+        for item_id, result, time_elapsed in item_results:
+            self.profiling_info.append((time_elapsed, item_id))
             if result == Item.STATUS_ACTION_SUCCEEDED:
                 self.correct += 1
             elif result == Item.STATUS_OK:
@@ -53,6 +65,9 @@ class ApplyResult(object):
                     "can't make sense of results for {} on {}: {}"
                 ).format(item_id, self.node_name, result))
 
+        self.profiling_info.sort()
+        self.profiling_info.reverse()
+
         self.start = None
         self.end = None
 
@@ -61,7 +76,7 @@ class ApplyResult(object):
         return self.end - self.start
 
 
-def apply_items(node, workers=1, interactive=False):
+def apply_items(node, workers=1, interactive=False, profiling=False):
     items = prepare_dependencies(node.items)
 
     with WorkerPool(workers=workers) as worker_pool:
@@ -156,7 +171,7 @@ def apply_items(node, workers=1, interactive=False):
                             print(formatted_result)
                         else:
                             LOG.info(formatted_result)
-                        yield (skipped_item.id, Item.STATUS_SKIPPED)
+                        yield (skipped_item.id, Item.STATUS_SKIPPED, msg['duration'])
                 else:
                     # if an item is applied successfully, all
                     # dependencies on it can be removed from the
@@ -194,7 +209,7 @@ def apply_items(node, workers=1, interactive=False):
                             ))
 
                 if item.ITEM_TYPE_NAME != 'dummy':
-                    yield (item.id, status_code)
+                    yield (item.id, status_code, msg['duration'])
 
                 # Finally, we have a new job queue. Thus, tell all idle
                 # workers to ask for work again.
@@ -347,7 +362,16 @@ class Node(object):
         for bundle_name in found_bundles + list(self._bundles):
             if bundle_name not in added_bundles:
                 added_bundles.append(bundle_name)
-                yield Bundle(self, bundle_name)
+                try:
+                    yield Bundle(self, bundle_name)
+                except NoSuchBundle:
+                    raise NoSuchBundle(_(
+                        "Node '{node}' wants bundle '{bundle}', but it doesn't exist."
+                    ).format(
+                        bundle=bundle_name,
+                        node=self.name,
+                    ))
+
 
     @cached_property
     def groups(self):
@@ -383,7 +407,7 @@ class Node(object):
             for item in bundle.items:
                 yield item
 
-    def apply(self, interactive=False, force=False, workers=4):
+    def apply(self, interactive=False, force=False, workers=4, profiling=False):
         self.repo.hooks.node_apply_start(
             self.repo,
             self,
@@ -398,6 +422,7 @@ class Node(object):
                     self,
                     workers=worker_count,
                     interactive=interactive,
+                    profiling=profiling,
                 ))
         except NodeAlreadyLockedException as e:
             if not interactive:
@@ -436,9 +461,13 @@ class Node(object):
     def metadata(self):
         m = {}
         group_order = _flatten_group_hierarchy(self.groups)
-        for group in group_order:
-            m.update(self.repo.get_group(group).metadata)
+        for group_name in group_order:
+            m.update(self.repo.get_group(group_name).metadata)
         m.update(self._node_metadata)
+        for group_name in group_order:
+            group = self.repo.get_group(group_name)
+            for metadata_processor in group.metadata_processors:
+                m = metadata_processor(self.name, group_order, m)
         return m
 
     @property
